"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const registry_1 = require("./registry");
const clean_1 = require("./tasks/clean");
const statics_1 = require("./tasks/statics");
const tslint_1 = require("./tasks/tslint");
const compiler_1 = require("./tasks/compiler");
const serverTest_1 = require("./tasks/serverTest");
const server_1 = require("./tasks/server");
const webpackDevServer_1 = require("./tasks/webpackDevServer");
class BasicWebAppTooling extends registry_1.Registry {
    init(gulp) {
        super.init(gulp);
        const { task, parallel, series } = gulp;
        const { inject } = this;
        const server = new server_1.Server(this.context);
        const cleaner = new clean_1.Cleaner(this.context);
        const statics = new statics_1.Statics(this.context);
        const tslint = new tslint_1.TsLinter(this.context);
        const compiler = new compiler_1.ServerCompiler(this.context);
        const serverTest = new serverTest_1.ServerTest(this.context, server);
        const webpackDevServer = new webpackDevServer_1.WebpackDevServer(this.context);
        task('clean:client', cleaner.cleanClient);
        task('clean:server', cleaner.cleanServer);
        task('clean', parallel(inject('clean:client'), inject('clean:server')));
        task('statics:public', statics.public);
        task('statics:json', statics.json);
        task('statics', parallel(inject('statics:public'), inject('statics:json')));
        task('tslint:tasks', tslint.lintTasks);
        task('tslint:server', tslint.lintServer);
        task('tslint:common', tslint.lintCommon);
        task('tslint:client', tslint.lintClient);
        task('tslint', parallel(inject('tslint:server'), inject('tslint:client'), inject('tslint:common')));
        task('client:prebuild', series(inject('clean:client'), inject('statics:public')));
        task('client:build', series(parallel(inject('tslint:client'), inject('client:prebuild'))
        /* TODO: add webpack here */ ));
        task('webpack:devServer', webpackDevServer.serve);
        task('client:dev', series(inject('client:prebuild'), inject('webpack:devServer')));
        task('server:precompile', inject('statics:json'));
        task('server:compile', series(inject('server:precompile'), compiler.compile));
        task('server:test:single', series(inject('server:compile'), serverTest.single));
        task('server:test', series(parallel(inject('server:test:single'), compiler.watch), serverTest.continuous));
        task('server:build', parallel(inject('tslint:server'), inject('server:compile')));
        task('server:run', series(inject('clean:server'), inject('server:compile'), parallel(compiler.watch, serverTest.applyHooks), server.serve));
        task('dev', parallel(inject('client:dev'), inject('server:run')));
        task('validate', parallel(inject('tslint'), inject('server:test:single')));
        task('default', inject('dev'));
    }
}
exports.BasicWebAppTooling = BasicWebAppTooling;
